//------------------------------------------------------------------------------
//
// THIS SOFTWARE IS PROVIDED AS IS. THE JAVAPOS WORKING GROUP MAKES NO
// REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE,
// EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT.  INDIVIDUAL OR CORPORATE MEMBERS OF THE JAVAPOS
// WORKING GROUP SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED AS A RESULT
// OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
// MICRBeanInfo.java - Bean information for the JavaPOS MICR
//    device control
//
//------------------------------------------------------------------------------

package jpos;

import java.beans.*;
import java.lang.reflect.*;

public class MICRBeanInfo
  extends SimpleBeanInfo
{
  public BeanDescriptor getBeanDescriptor()
  {
    return new BeanDescriptor(jpos.MICR.class);
  }

  public PropertyDescriptor makeProperty(String propertyName)
    throws IntrospectionException
  {
    return new PropertyDescriptor(propertyName, jpos.MICR.class);
  }

  public PropertyDescriptor[] getPropertyDescriptors()
  {
    try
    {
      PropertyDescriptor[] properties =
      {
        // Capabilities
        makeProperty("CapValidationDevice"),
        makeProperty("CapPowerReporting"),

        // Properties
        makeProperty("CheckHealthText"),
        makeProperty("Claimed"),
        makeProperty("DeviceControlDescription"),
        makeProperty("DeviceControlVersion"),
        makeProperty("DeviceEnabled"),
        makeProperty("DeviceServiceDescription"),
        makeProperty("DeviceServiceVersion"),
        makeProperty("FreezeEvents"),
        makeProperty("PhysicalDeviceDescription"),
        makeProperty("PhysicalDeviceName"),
        makeProperty("State"),
        makeProperty("AutoDisable"),
        makeProperty("AccountNumber"),
        makeProperty("Amount"),
        makeProperty("BankNumber"),
        makeProperty("CheckType"),
        makeProperty("CountryCode"),
        makeProperty("DataCount"),
        makeProperty("DataEventEnabled"),
        makeProperty("EPC"),
        makeProperty("RawData"),
        makeProperty("SerialNumber"),
        makeProperty("TransitNumber"),
        makeProperty("PowerNotify"),
        makeProperty("PowerState")
      };

      return properties;
    }
    catch(Exception e)
    {
      return super.getPropertyDescriptors();
    }
  }

  public EventSetDescriptor makeEvent(String eventName)
    throws IntrospectionException, ClassNotFoundException
  {
    String listener = "jpos.events." + eventName + "Listener";
    return new EventSetDescriptor(jpos.MICR.class,
                                  eventName,
                                  Class.forName(listener),
                                  eventName + "Occurred");
  }

  public EventSetDescriptor[] getEventSetDescriptors()
  {
    try
    {
      EventSetDescriptor[] events =
      {
        makeEvent("Data"),
        makeEvent("DirectIO"),
        makeEvent("Error"),
        makeEvent("StatusUpdate")
      };

      return events;
    }
    catch(Exception e)
    {
      return super.getEventSetDescriptors();
    }
  }
}
